/*
 * Created on 26.09.2005
 *
 * $Id: InclusionDependency.java,v 1.4 2006/08/11 11:12:26 forberg Exp $
 */
package de.huberlin.informatik.wbi.aladin.intraschema;

import java.io.Serializable;

import de.huberlin.informatik.wbi.aladin.util.*;

/**
 * 
 * Represents a unary inclusion dependency.
 * 
 * @author forberg
 *
 */
public class InclusionDependency implements Serializable{

	private static final long serialVersionUID = -319493570453611178L;
	
    private String schemaName;
    private String dependentTable;
    private String dependentColumn;
    private String referencedTable;
    private String referencedColumn;
    
    
    
    public InclusionDependency(
            String schemaName, 
            String dependentTable, String dependentColumn, 
            String referencedTable, String referencedColumn) {
        
        this.schemaName = schemaName;
        this.dependentTable = dependentTable;
        this.dependentColumn = dependentColumn;
        this.referencedTable = referencedTable;
        this.referencedColumn = referencedColumn;
    }

    public InclusionDependency(
            String dependentTable, String dependentColumn, 
            String referencedTable, String referencedColumn) {
        
        this(null, dependentTable, dependentColumn, referencedTable, referencedColumn);
    }
            
    
    public InclusionDependency(
            ColumnData dependent, 
            String referencedTable, String referencedColumn) {
        
        this(null, dependent.getTableName(), dependent.getColumnName(), 
                referencedTable, referencedColumn);
        
    }

    public InclusionDependency(ColumnData dependent, ColumnData referenced) {
        
        this(dependent.getSchemaName(), dependent.getTableName(), dependent.getColumnName(), 
                referenced.getTableName(), referenced.getColumnName());
        
        if ( !dependent.getSchemaName().equals(referenced.getSchemaName()) ) {
            throw new AladinException("columns of different schemas given: " 
                    + "dependent schema = " + dependent.getSchemaName() 
                    + ", referenced schema = " + referenced.getSchemaName());
        }
        
    }
    
    public InclusionDependency(String schemaName, ColumnInfo dependent, ColumnInfo referenced) {
        
        this(schemaName, dependent.getTableName(), dependent.getColumnName(), 
                referenced.getTableName(), referenced.getColumnName());
    }
    
    public InclusionDependency(String dependent, String referenced) {
        
        String[] dependentSplitted = dependent.split("\\.");
        String[] referencedSplitted = referenced.split("\\.");
        
        this.schemaName = null;
        
        this.dependentTable = dependentSplitted[0];
        this.dependentColumn = dependentSplitted[1];
        
        this.referencedTable = referencedSplitted[0];
        this.referencedColumn = referencedSplitted[1];
        
    }
    
    public boolean equals(Object o) {
        
        if ( !(o instanceof InclusionDependency) )
            return false;
        
        InclusionDependency other = (InclusionDependency) o;
        
        if ( this.schemaName == null 
                ? other.schemaName != null 
                : !this.schemaName.equals(other.schemaName))
            return false;
        
        if ( !this.dependentTable.equals(other.dependentTable) )
            return false;
        
        if ( !this.dependentColumn.equals(other.dependentColumn) )
            return false;
        
        if ( !this.referencedTable.equals(other.referencedTable) )
            return false;
        
        if ( !this.referencedColumn.equals(other.referencedColumn) )
            return false;
        
        // passed all tests
        return true;
        
    }
    
    public int hashCode() {
        
        int hashCode = 0;
        hashCode += dependentTable.hashCode();
        hashCode += dependentColumn.hashCode();
        hashCode += referencedTable.hashCode();
        hashCode += referencedColumn.hashCode();
        
        return hashCode;
    }
    
    public String getDependentColumn() {
        return dependentColumn;
    }

    public String getDependentTable() {
        return dependentTable;
    }

    public String getReferencedColumn() {
        return referencedColumn;
    }

    public String getReferencedTable() {
        return referencedTable;
    }

    public String getSchemaName() {
        return schemaName;
    }

    public void setSchemaName(String schemaName) {
        this.schemaName = schemaName;
    }
    
    public ColumnInfo getDependentAtt() {
        return new ColumnInfo(schemaName, dependentTable, dependentColumn);
    }
    
    public ColumnInfo getReferencedAtt() {
        return new ColumnInfo(schemaName, referencedTable, referencedColumn);
    }
    
    /**
     * @see Object#toString()
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();

        sb.append( dependentTable)
            .append(".")
            .append(dependentColumn)
            .append(" -> ")
            .append(referencedTable)
            .append(".")
            .append(referencedColumn);

        return sb.toString();
    }
}
