/**
 * This GUI shows the Result of FKFinder as a list, showing all IND's and the resultvalue 
 * for each. The GUI must be opended from the Main-GUi or from the Progress GUI.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Map;
import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKeyFinderExec;
import de.huberlin.informatik.wbi.aladin.util.ForeignKeyFinderResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class FkFinderGuiResult extends Thread{

	private Button saveResultButton;
	protected Shell shell;
	
	/**
	 * Loaded results are stored here.
	 */
	private ForeignKeyFinderExec finder;
	private boolean getresultFromFinder = false;
	private Vector<ForeignKey> result;
	private boolean getResultFromVector = false;

	/**
	 * Launch the application
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			FkFinderGuiResult window = new FkFinderGuiResult();
			window.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Open the window
	 */
	public void open() {
		final Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}
	
	/**
	 * Start the window as Thread.
	 */
	public void run()
	{
		this.open();
	}
	
	/**
	 * Saves a ForeignKeyfinderExcec object to get the results from.
	 * @param finder
	 */
	public void setResult(ForeignKeyFinderExec finder)
	{
		this.finder = finder;
		this.getresultFromFinder = true;
	}
	
	/**
	 * Saves the result as vector object holding the ForeignKey objects.
	 * @param result
	 */
	public void setResult(Vector<ForeignKey> result)
	{
		this.result = result;
		this.getResultFromVector = true;
	}
	
	/**
	 * The showed result is saved into a file. It will be created a new file 
	 * in the directory "result" with such name: ForeignKeyFinderResult<counter>
	 */
	public void saveResult()
	{
		try
		{
			// find Filename
			int fileCounter = 0;
			while((new File("result/ForeignKeyFinderResult" + fileCounter)).exists())
			{
				fileCounter++;
			}
			FileOutputStream ostream = new FileOutputStream("result/ForeignKeyFinderResult" + fileCounter);
			ObjectOutputStream output = new ObjectOutputStream(ostream);
			output.writeObject(new ForeignKeyFinderResult(this.finder.getConfig(), 
					this.finder.getResult()));
			output.flush();
			MessageBox alert = new MessageBox(shell, SWT.OK);
			alert.setMessage("Resultset was saved to: result/ForeignKeyFinderResult" + fileCounter);
			alert.open();
			saveResultButton.setEnabled(false);
		}
		catch (Exception e)
		{
			MessageBox alert = new MessageBox(shell, SWT.OK);
			alert.setMessage("Error while saving result: " + e.getMessage());
			alert.open();
			e.printStackTrace();
		}
		
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		shell = new Shell();
		shell.setLayout(new FormLayout());
		shell.setSize(800, 408);
		shell.setText("FKFinder - results");

		final Tree tree = new Tree(shell, SWT.BORDER);
		final FormData fd_tree = new FormData();
		fd_tree.right = new FormAttachment(100, -16);
		fd_tree.bottom = new FormAttachment(100, -45);
		fd_tree.top = new FormAttachment(0, 10);
		fd_tree.left = new FormAttachment(0, 10);
		tree.setLayoutData(fd_tree);
		tree.setHeaderVisible(true);

		final TreeColumn newColumnTreeColumn = new TreeColumn(tree, SWT.LEFT);
		newColumnTreeColumn.setWidth(450);
		newColumnTreeColumn.setText("IND / Heuristic");

		final TreeColumn newColumnTreeColumn_1 = new TreeColumn(tree, SWT.NONE);
		newColumnTreeColumn_1.setWidth(330);
		newColumnTreeColumn_1.setText("Result");

		
		saveResultButton = new Button(shell, SWT.NONE);
		saveResultButton.setToolTipText("Save the shown result to disk");
		saveResultButton.addMouseListener(new MouseAdapter() {
			public void mouseDown(final MouseEvent e) {
					saveResult();	
			}
		});
		if(this.getResultFromVector || (this.getResultFromVector == false && this.getresultFromFinder == false))
		{
			saveResultButton.setEnabled(false);
		}
		final FormData fd_saveResultButton = new FormData();
		fd_saveResultButton.left = new FormAttachment(42, 0);
		fd_saveResultButton.right = new FormAttachment(57, 0);
		fd_saveResultButton.top = new FormAttachment(tree, 8, SWT.DEFAULT);
		saveResultButton.setLayoutData(fd_saveResultButton);
		saveResultButton.setText("Save Result");

		
		if(this.getresultFromFinder || this.getResultFromVector)
        {
        	// Putting Data into GUI
			Vector<ForeignKey> result;
			if(this.getresultFromFinder)
			{
				result = this.finder.getResult();
			}
			else
			{
				result = this.result;
			}
			Collections.sort(result);
			for(ForeignKey key : result)
			{
				TreeItem item = new TreeItem(tree, SWT.NONE);
				item.setText(new String[] { key.toString(), key.getProbability() + "/" 
						+ key.getMaxProbability() + " -> " + key.getProbabilityPercentage() + "%"});
				// heuristics
				for(Map.Entry entry : key.getHeuriticsResult().entrySet())
				{
					TreeItem subItem = new TreeItem(item, SWT.NONE);
			        subItem.setText(new String[] { (String)entry.getKey() + ":", 
			        		((HeuristicResult)entry.getValue()).toString()});
				}
			}
        }
		
	}

}
