/**
 * The main GUI for FKFinder. From this GUI, existing configurations can be loaded
 * or stored results can be loaded and put into another GUI. 
 */
package de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.MessageBox;

import de.huberlin.informatik.wbi.aladin.util.ForeignKeyFinderResult;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;
import java.util.Vector;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.FileWriter;

import javax.naming.ldap.Control;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class FkfinderGui {

	private Combo combo;
	private Combo Loglevel;
	private Button printResultToShell;
	private Button saveResultInDatabase;
	private Button configMMCheckColumnInfo;
	private Button configViACheckColumnInfo;
	private Text dbConnect;
	private Button minMaxButton;
	private Button columnNameButton;
	private Button numberOfDependenciesButton;
	private Button coveringButton;
	private Button numberOfValuesButton;
	private Text spiderOutTable;
	private Combo configurations;
	private Text configMMValuesOutside;
	private Text configNoDMax;
	private Text configCoveringFalse;
	private Text configCoveringMin;
	private Text configMinNumberOfValues;
	private Text EmphasisMinMax;
	private Text EmphasisCloumnName;
	private Text EmphasisNumberOfDep;
	private Text EmphasiseCovering;
	private Text EmphasisNumberOfValues;
	private Text spiderOutSchema;
	private Text schema;
	private Text dbTable;
	private Text dbPassword;
	private Text dbUser;
	protected Shell shell;
	
	/**
	 * The date format that is used in the DropDown for the stored results.
	 */
	private final SimpleDateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	
	/**
	 * Logger for logging
	 */
	static Logger logger = Logger.getLogger(FkfinderGui.class.getName());

	/**
	 * Launch the application, can be startet from the shell.
	 * @param args
	 */
	public static void main(String[] args) {
		logger.setLevel(Level.DEBUG);
		
		try {
			FkfinderGui window = new FkfinderGui();
			window.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Open the window.
	 */
	public void open() {
		final Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		shell = new Shell();
		shell.setSize(507, 722);
		shell.setText("FKFinder - main GUI");

		configurations = new Combo(shell, SWT.NONE);
		configurations.setToolTipText("Load a configuration that is stored in cfg directory");
		configurations.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) 
			{
				try
				{
					putConfigurationByText((configurations.getText()));
				}
				catch (IllegalArgumentException ex)
				{
					MessageBox alert = new MessageBox(shell, SWT.OK);
					alert.setMessage("Invalid configuration.");
					alert.open();
					return;
				}
			}
		});
		configurations.setBounds(141, 8, 350, 21);

		// put an item for every configuration
		configurations.setItems(this.getConfigurationNames());
		
		final TabFolder tabFolder = new TabFolder(shell, SWT.NONE);
		tabFolder.setBounds(0, 41, 507, 615);

		final TabItem testTabItem = new TabItem(tabFolder, SWT.NONE);
		testTabItem.setToolTipText("General configuration");
		testTabItem.setText("general");

		final Composite composite = new Composite(tabFolder, SWT.NONE);
		testTabItem.setControl(composite);

		final Group databaseGroup = new Group(composite, SWT.NONE);
		databaseGroup.setBounds(10, 10, 481, 117);
		databaseGroup.setText("Database");

		dbUser = new Text(databaseGroup, SWT.BORDER);
		dbUser.setToolTipText("Username for database connection");
		dbUser.setBounds(234, 15,237, 19);

		dbPassword = new Text(databaseGroup, SWT.BORDER);
		dbPassword.setToolTipText("Password for database connection");
		dbPassword.setBounds(234, 40,237, 19);

		dbTable = new Text(databaseGroup, SWT.BORDER);
		dbTable.setToolTipText("Table holding");
		dbTable.setBounds(234, 65,237, 19);

		dbConnect = new Text(databaseGroup, SWT.BORDER);
		dbConnect.setBounds(235, 90, 237, 19);

		final Label tableNameLabel = new Label(databaseGroup, SWT.NONE);
		tableNameLabel.setToolTipText("Name of the schema and table where the columninfo information are stored");
		tableNameLabel.setBounds(10, 71,180, 13);
		tableNameLabel.setText("Column_infos table");

		final Label userLabel = new Label(databaseGroup, SWT.NONE);
		userLabel.setBounds(10, 43,186, 13);
		userLabel.setText("Password");

		final Label schemaLabel = new Label(databaseGroup, SWT.NONE);
		schemaLabel.setBounds(10, 18,134, 13);
		schemaLabel.setText("User");

		final Label connectString = new Label(databaseGroup, SWT.NONE);
		connectString.setToolTipText("Connect string for jdbc driver");
		connectString.setBounds(10, 96,180, 13);
		connectString.setText("Connect String");

		final Group schemaToCheckGroup = new Group(composite, SWT.NONE);
		schemaToCheckGroup.setText("Schema to check");
		schemaToCheckGroup.setBounds(10, 133, 481, 71);

		schema = new Text(schemaToCheckGroup, SWT.BORDER);
		schema.setToolTipText("Name of the Schema that should be checked");
		schema.setBounds(234, 15, 237, 19);

		spiderOutSchema = new Text(schemaToCheckGroup, SWT.BORDER);
		spiderOutSchema.setToolTipText("Name of the schema where the SPIDER output is stored");
		spiderOutSchema.setBounds(234, 40, 112, 19);

		spiderOutTable = new Text(schemaToCheckGroup, SWT.BORDER);
		spiderOutTable.setToolTipText("Name of the table where the SPIDER output is tored");
		spiderOutTable.setBounds(355, 40, 112, 19);

		final Label schemaLabel_1 = new Label(schemaToCheckGroup, SWT.NONE);
		schemaLabel_1.setText("Schema");
		schemaLabel_1.setBounds(10, 18, 159, 13);

		final Label spiderOutputLabel = new Label(schemaToCheckGroup, SWT.NONE);
		spiderOutputLabel.setText("Spider Output");
		spiderOutputLabel.setBounds(10, 43, 112, 13);

		final Label label = new Label(schemaToCheckGroup, SWT.NONE);
		label.setText(".");
		label.setBounds(349, 45, 3, 14);

		final Group heuristicsGroup = new Group(composite, SWT.NONE);
		heuristicsGroup.setText("Heuristics");
		heuristicsGroup.setBounds(10, 210, 479, 163);

		numberOfDependenciesButton = new Button(heuristicsGroup, SWT.CHECK);
		numberOfDependenciesButton.setToolTipText("Select if this heuristic should be used");
		numberOfDependenciesButton.setText("Number of references");
		numberOfDependenciesButton.setBounds(10, 38, 219, 16);

		EmphasisNumberOfDep = new Text(heuristicsGroup, SWT.BORDER);
		EmphasisNumberOfDep.setToolTipText("Emphasis for this heuristic in the overall result");
		EmphasisNumberOfDep.setBounds(235, 35, 237, 19);

		numberOfValuesButton = new Button(heuristicsGroup, SWT.CHECK);
		numberOfValuesButton.setToolTipText("Select if this heuristic should be used");
		numberOfValuesButton.setText("Number of dependend values");
		numberOfValuesButton.setBounds(10, 65, 225, 16);

		EmphasiseCovering = new Text(heuristicsGroup, SWT.BORDER);
		EmphasiseCovering.setToolTipText("Emphasis for this heuristic in the overall result");
		EmphasiseCovering.setBounds(235, 60, 237, 19);

		coveringButton = new Button(heuristicsGroup, SWT.CHECK);
		coveringButton.setToolTipText("Select if this heuristic should be used");
		coveringButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
			}
		});
		coveringButton.setText("Covering");
		coveringButton.setBounds(10, 88, 162, 16);

		EmphasisNumberOfValues = new Text(heuristicsGroup, SWT.BORDER);
		EmphasisNumberOfValues.setToolTipText("Emphasis for this heuristic in the overall result");
		EmphasisNumberOfValues.setBounds(235, 85, 237, 19);

		columnNameButton = new Button(heuristicsGroup, SWT.CHECK);
		columnNameButton.setToolTipText("Select if this heuristic should be used");
		columnNameButton.setText("Column name");
		columnNameButton.setBounds(10, 110, 195, 16);

		EmphasisCloumnName = new Text(heuristicsGroup, SWT.BORDER);
		EmphasisCloumnName.setToolTipText("Emphasis for this heuristic in the overall result");
		EmphasisCloumnName.setBounds(235, 110, 237, 19);

		minMaxButton = new Button(heuristicsGroup, SWT.CHECK);
		minMaxButton.setToolTipText("Select if this heuristic should be used");
		minMaxButton.setText("Spreading over reference");
		minMaxButton.setBounds(10, 136, 225, 16);

		EmphasisMinMax = new Text(heuristicsGroup, SWT.BORDER);
		EmphasisMinMax.setToolTipText("Emphasis for this heuristic in the overall result");
		EmphasisMinMax.setBounds(235, 135, 237, 19);

		final Label emphasisLabel = new Label(heuristicsGroup, SWT.NONE);
		emphasisLabel.setText("Emphasis:");
		emphasisLabel.setBounds(235, 15, 228, 13);

		final Group heuristicConfigurationGroup = new Group(composite, SWT.NONE);
		heuristicConfigurationGroup.setText("Heuristic configuration");
		heuristicConfigurationGroup.setBounds(10, 379, 479, 197);

		configMinNumberOfValues = new Text(heuristicConfigurationGroup, SWT.BORDER);
		configMinNumberOfValues.setToolTipText("The minimum of different values that have to be in the dependend attribut");
		configMinNumberOfValues.setBounds(235, 18, 237, 19);

		configCoveringMin = new Text(heuristicConfigurationGroup, SWT.BORDER);
		configCoveringMin.setToolTipText("Minimum percentage value of the covering between dep and ref to return TRUE");
		configCoveringMin.setBounds(235, 43, 237, 19);

		configCoveringFalse = new Text(heuristicConfigurationGroup, SWT.BORDER);
		configCoveringFalse.setToolTipText("If the covering between dep. and ref. attribut is below this value, FALSE is returned");
		configCoveringFalse.setBounds(235, 68, 237, 19);

		configNoDMax = new Text(heuristicConfigurationGroup, SWT.BORDER);
		configNoDMax.setToolTipText("Maximum number of dep. attributs that refere to one ref attribut");
		configNoDMax.setBounds(235, 93, 237, 19);

		configMMValuesOutside = new Text(heuristicConfigurationGroup, SWT.BORDER);
		configMMValuesOutside.setToolTipText("Percent of values of the ref. attribut that are allowed outside the boirder given by min() max() of the dep. attribut");
		configMMValuesOutside.setBounds(235, 118, 237, 19);

		configViACheckColumnInfo = new Button(heuristicConfigurationGroup, SWT.CHECK);
		configViACheckColumnInfo.setToolTipText("Select if table cloumninfo should be used for NumberOfValues heuristic");
		configViACheckColumnInfo.setBounds(235, 143, 237, 21);

		configMMCheckColumnInfo = new Button(heuristicConfigurationGroup, SWT.CHECK);
		configMMCheckColumnInfo.setToolTipText("Select if table cloumninfo should be used for MinMax heuristic");
		configMMCheckColumnInfo.setBounds(235, 165, 23, 21);

		final Label numberofvaluesminnumberofvaluesLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		numberofvaluesminnumberofvaluesLabel.setText("NumberDependendValues.minCount");
		numberofvaluesminnumberofvaluesLabel.setBounds(10, 21, 219, 13);

		final Label numberofvaluescheckcolumninfoLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		numberofvaluescheckcolumninfoLabel.setText("NOfDepValues.CheckColumnInfos");
		numberofvaluescheckcolumninfoLabel.setBounds(10, 145, 220, 13);

		final Label coveringpercentagecoveringminLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		coveringpercentagecoveringminLabel.setText("Covering.PercentageCoveringMin");
		coveringpercentagecoveringminLabel.setBounds(10, 47, 219, 13);

		final Label coveringpercentagecoveringfalseLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		coveringpercentagecoveringfalseLabel.setText("Covering.PercentageCoveringFalse");
		coveringpercentagecoveringfalseLabel.setBounds(10, 70, 209, 13);

		final Label numberofdependenciesmaxcountLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		numberofdependenciesmaxcountLabel.setText("NumberOfReferences.MaxCount");
		numberofdependenciesmaxcountLabel.setBounds(10, 95, 216, 13);

		final Label minmaxcheckcolumninfoLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		minmaxcheckcolumninfoLabel.setText("SpreadingOverRef.CheckColumnInfos");
		minmaxcheckcolumninfoLabel.setBounds(10, 170, 219, 13);

		final Label minmaxvaluesoutsideLabel = new Label(heuristicConfigurationGroup, SWT.NONE);
		minmaxvaluesoutsideLabel.setText("SpreadingOverRef.ValuesOutside");
		minmaxvaluesoutsideLabel.setBounds(10, 120, 209, 14);

		final TabItem extendedTabItem = new TabItem(tabFolder, SWT.NONE);
		extendedTabItem.setToolTipText("Extended configuration");
		extendedTabItem.setText("extended");

		final Composite composite_1 = new Composite(tabFolder, SWT.NONE);
		extendedTabItem.setControl(composite_1);

		saveResultInDatabase = new Button(composite_1, SWT.CHECK);
		saveResultInDatabase.setToolTipText("Enable if the result should be saved into the database");
		saveResultInDatabase.setText("Save Result in Database?");
		saveResultInDatabase.setBounds(10, 10, 190, 21);

		printResultToShell = new Button(composite_1, SWT.CHECK);
		printResultToShell.setToolTipText("Enable if the result should be printed to the shel");
		printResultToShell.setAlignment(SWT.LEFT);
		printResultToShell.setText("Print result to shell?");
		printResultToShell.setBounds(10, 37, 190, 21);
		final Label loglevelLabel = new Label(composite_1, SWT.NONE);

		loglevelLabel.setText("Loglevel");
		loglevelLabel.setBounds(260, 13, 56, 14);

		Loglevel = new Combo(composite_1, SWT.NONE);
		Loglevel.setToolTipText("Choose a loglevel that is used for the Logger");
		Loglevel.setItems(new String[] {"ALL", "DEBUG", "INFO", "WARN", "ERROR", "FATAL", "OFF"});
		Loglevel.setBounds(320, 5, 100, 27);

		final TabItem loadResultsetTabItem = new TabItem(tabFolder, SWT.NONE);
		loadResultsetTabItem.setToolTipText("Load a stored result");
		loadResultsetTabItem.setText("load resultset");

		final Composite composite_2 = new Composite(tabFolder, SWT.NONE);
		loadResultsetTabItem.setControl(composite_2);

		combo = new Combo(composite_2, SWT.NONE);
		combo.setToolTipText("Choose a result that is stored in directory result");
		combo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				loadResult(combo.getSelectionIndex());
			}
		});
		combo.setBounds(10, 10, 411, 27);
		this.updateResultSets();

		final Button refreshButton = new Button(composite_2, SWT.NONE);
		refreshButton.setToolTipText("Refresh the list and read the stored files again");
		refreshButton.addMouseListener(new MouseAdapter() {
			public void mouseDown(final MouseEvent e) {
				updateResultSets();
			}
		});
		refreshButton.setText("refresh");
		refreshButton.setBounds(430, 10, 63, 26);

		final Button startButton = new Button(shell, SWT.NONE);
		startButton.setToolTipText("Start the Finder");
		startButton.addMouseListener(new MouseAdapter() {
			public void mouseDown(final MouseEvent e) {
				try {
					startFkFinder();
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}
		});
		startButton.setText("Start");
		startButton.setBounds(370, 665, 121, 23);

		final Label loadConfigurationLabel = new Label(shell, SWT.NONE);
		loadConfigurationLabel.setText("Load configuration");
		loadConfigurationLabel.setBounds(10, 11, 120, 13);
		
		final Button startButton_1 = new Button(shell, SWT.NONE);
		startButton_1.setToolTipText("Save this configuration to disk");
		startButton_1.addMouseListener(new MouseAdapter() {
			public void mouseDown(final MouseEvent e) {
				saveConfiguration();
			}
		});
		startButton_1.setBounds(15, 665, 121, 23);
		startButton_1.setText("Save");

	}
	
	/**
	 * Updates the Combo with all available result sets.
	 */
	private void updateResultSets()
	{
		combo.setItems(getResultnames());
	}
	
	/**
	 * Loads a stored result. The configuration is loaded into the Mein Gui,
	 * the result window is opened with the Inclusion dependencies that are stored in
	 * the Resultset.
	 * @param selectedIndex Index of the selected result.
	 */
	private void loadResult(int selectedIndex)
	{
		try
		{
			ForeignKeyFinderResult result = this.getResultsets().elementAt(selectedIndex);
			this.loadConfigurationToGui(result.getPropertie());
			FkFinderGuiResult resultGui = new FkFinderGuiResult();
			resultGui.setResult(result.getResult());
			resultGui.open();
		}
		catch (Exception e)
		{
			MessageBox alert = new MessageBox(shell, SWT.OK);
			alert.setMessage("Error while loading result: " + e.getMessage());
			alert.open();
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns all the available configuration.
	 * 
	 * @return Vector of the available configuration files.
	 */
	private Vector<Properties> getconfigurations()
	{
		// Read out all Files in the cfg Directory
		File f = new File("cfg");
		File[] fileArray = f.listFiles();
		
		Vector<Properties> properties = new Vector<Properties>();
				
		//check for each file if it is a configuration file
		if (fileArray != null) 
		{
			for (int i = 0; i < fileArray.length; i++) 
			{
		         if (!fileArray[i].isDirectory()) 
		         {
		        	 Properties prop = new Properties();
		        	 try
		        	 {
		        		 prop.load(new FileInputStream(fileArray[i].getAbsolutePath()));
		        		 if(prop.getProperty("ConfigurationName").length() > 0)
		        		 {
		        			 prop.setProperty("FileName", fileArray[i].getPath());
		        			 properties.add(prop);
		        		 }
		        	 }
		        	 catch (Exception e)
		        	 {
		        		 
		        	 }
		         }
			}
		}
		
		return properties;
	}
	
	/**
	 * Reads all available Resultsets that where save before.
	 * @return Resultsets as Vector
	 */
	private Vector<ForeignKeyFinderResult> getResultsets()
	{
		File f = new File("result");
		File[] fileArray = f.listFiles();
		
		Vector<ForeignKeyFinderResult> resultSets = new Vector<ForeignKeyFinderResult>();
				
		//check for each file if it is a configuration file
		if (fileArray != null) 
		{
			for (int i = 0; i < fileArray.length; i++) 
			{
		         if (!fileArray[i].isDirectory()) 
		         {
		        	 try
		        	 {
		        		 FileInputStream istream = new FileInputStream(fileArray[i].getAbsoluteFile());
		        		 ObjectInputStream input = new ObjectInputStream(istream);
		        		 ForeignKeyFinderResult rset = ((ForeignKeyFinderResult) input.readObject());
		        		 
		        		 Properties prop = rset.getPropertie();
		        		 prop.setProperty("lastModified", new Long(fileArray[i].lastModified()).toString());
		        		 if(prop.getProperty("ConfigurationName").length() > 0)
		        		 {
		        			 resultSets.add(rset);
		        		 }
		        	 }
		        	 catch (Exception e)
		        	 {
		        		 System.out.println("Could not read" + fileArray[i].getAbsoluteFile());
		        		 e.printStackTrace();
		        	 }
		         }
			}
		}
		return resultSets;
	}
	
	/**
	 * Returns the Names of all available configurations.
	 * 
	 * @return stringarray of the configurationnames
	 */
	private String[] getConfigurationNames()
	{
		Vector<String> names = new Vector<String>();
		for(Properties pro : this.getconfigurations())
		{
			names.add(pro.getProperty("ConfigurationName"));
		}
		return names.toArray(new String[0]);
	}
	
	/**
	 * Resturns the ResultsetNames of all available Results
	 * @return
	 */
	private String[] getResultnames()
	{
		Vector<ForeignKeyFinderResult> resultSets = this.getResultsets();
		Vector<String> name = new Vector<String>();
		for(ForeignKeyFinderResult res : resultSets)
		{
			Properties prop = res.getPropertie();
			Date datum = new Date(new Long(prop.getProperty("lastModified")));
			name.add(prop.getProperty("ConfigurationName") + " (" + dateFormat.format( datum ) + ")");
		}
		
		return name.toArray(new String[0]);
	}
	
	/**
	 * Opens the progress window where the finder is started from.
	 */
	private void startFkFinder()
	{
		FkfinderGuiWorking window = new FkfinderGuiWorking();
		Properties config = new Properties();
		this.getPropertiesFromGui(config);
		
		window.open(true, config);
	}
	
	/**
	 * Loads the configuration from a file into the GUI
	 *
	 * @param configName The Name of the Configuration
	 */
	private void putConfigurationByText(String configName)
	{
		Properties config = null;
		
		// getConfiguration By Name
		for(Properties pro : this.getconfigurations())
		{
			if(pro.getProperty("ConfigurationName").equals(configName))
			{
				config = pro;
				break;
			}
		}
		// Putting config values into GUI
		if(config != null)
		{
			this.loadConfigurationToGui(config);
		}
	}
	
	/**
	 * Loads a configuration from a given Properties object into the GUI elements
	 * @param config
	 */
	private void loadConfigurationToGui(Properties config)
	{
		// Database
		dbTable.setText(config.getProperty("Database.columnInfoTable"));
		dbPassword.setText(config.getProperty("Database.password"));
		dbUser.setText(config.getProperty("Database.user"));
		dbConnect.setText(config.getProperty("Database.connectString"));
		
		// Schema to check
		schema.setText(config.getProperty("ForeignKeyFinder.checkSchema"));
		spiderOutSchema.setText(config.getProperty("ForeignKeyFinder.schemaName"));
		spiderOutTable.setText(config.getProperty("ForeignKeyFinder.tableName"));
		
		// Heuristics and emphasises
		String[] emphasis = config.getProperty(
			"ForeignKeyFinder.HeuristicsEmphasis").split(",");
		String[] heuristics = config.getProperty(
			"ForeignKeyFinder.HeuristicsToUse").split(",");
		if(emphasis.length != heuristics.length)
		{
			logger.fatal("Wrong Configuration: Number of " +
					"Heuristics differs to number of emphasises");
			System.exit(5);
		}
		for(int i = 0; i < heuristics.length; i++)
		{
			if(heuristics[i].equals("ForeignKeyHeuristicMinNumberDependendValues"))
			{
				EmphasisNumberOfValues.setText(emphasis[i]);
			}
			else if(heuristics[i].equals("ForeignKeyHeuristicCovering"))
			{
				EmphasiseCovering.setText(emphasis[i]);
			}
			else if(heuristics[i].equals("ForeignKeyHeuristicNumberOfReferences"))
			{
				EmphasisNumberOfDep.setText(emphasis[i]);
			}
			else if(heuristics[i].equals("ForeignKeyHeuristicColumnName"))
			{
				EmphasisCloumnName.setText(emphasis[i]);
			}
			else if(heuristics[i].equals("ForeignKeyHeuristicSpreadingOverReference"))
			{
				EmphasisMinMax.setText(emphasis[i]);
			}
		}
		numberOfValuesButton.setSelection(config.getProperty("ForeignKeyFinder.HeuristicsToUse").
				contains("ForeignKeyHeuristicMinNumberDependendValues"));
		coveringButton.setSelection(config.getProperty("ForeignKeyFinder.HeuristicsToUse").
				contains("ForeignKeyHeuristicCovering"));
		numberOfDependenciesButton.setSelection(config.getProperty("ForeignKeyFinder.HeuristicsToUse").
				contains("ForeignKeyHeuristicNumberOfReferences"));
		columnNameButton.setSelection(config.getProperty("ForeignKeyFinder.HeuristicsToUse").
				contains("ForeignKeyHeuristicColumnName"));
		minMaxButton.setSelection(config.getProperty("ForeignKeyFinder.HeuristicsToUse").
				contains("ForeignKeyHeuristicSpreadingOverReference"));
		
		// Heuristic configuration
		configMinNumberOfValues.setText(
				config.getProperty("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.minNumberOfValues"));
		configCoveringMin.setText(
				config.getProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringMin"));
		configCoveringFalse.setText(
				config.getProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringFalse"));
		configNoDMax.setText(
				config.getProperty("Heuristics.ForeignKeyHeuristicNumberOfReferences.MaxCount"));
		configMMCheckColumnInfo.setSelection(new Boolean(
				config.getProperty("Heuristics.ForeignKeyHeuristicSpreadingOverReference.CheckColumnInfo")));
		configViACheckColumnInfo.setSelection(new Boolean(
				config.getProperty("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.CheckColumnInfo")));
		configMMValuesOutside.setText(
				config.getProperty("Heuristics.ForeignKeyHeuristicSpreadingOverReference.ValuesOutsideMinMaxPercentage"));
		
		// extended configuration
		saveResultInDatabase.setSelection(new Boolean(
				config.getProperty("ForeignKeyFinder.writeResultIntoDatabase")));
		printResultToShell.setSelection(new Boolean(
				config.getProperty("ForeignKeyFinder.printResultsToShell")));
		
		for(int i = 0;i < Loglevel.getItems().length; i++)
		{
			if(Loglevel.getItems()[i].equals(config.getProperty("Logger.Level")))
			{
				Loglevel.select(i);
				break;
			}
		}
	}
	
	/**
	 * Saves the current configuration to a file.
	 * If the file does not exists, it will be created, otherwise it will be replaced.
	 */
	private void saveConfiguration()
	{
		// If no Name is set -> return
		MessageBox alert = null;
		
		if(configurations.getText().length() == 0)
		{
			alert = new MessageBox(shell, SWT.OK);
			alert.setMessage("Please set a Name for this configuration");
			alert.open();
			return;
		}

		// Check if configuration already exists
		Properties config = null;
		boolean found = false;
		for(Properties prop : this.getconfigurations())
		{
			if(prop.getProperty("ConfigurationName").equals(configurations.getText()))
			{
				config = prop;
				found = true;
				break;
			}
		}
		if(found == false)
		{
			config = new Properties();
			config.setProperty("ConfigurationName", configurations.getText());
			// find Filename
			int fileCounter = 0;
			while((new File("cfg/ForeignKeyFinder" + fileCounter + ".properties")).exists())
			{
				fileCounter++;
			}
			config.setProperty("FileName", "cfg/ForeignKeyFinder" + fileCounter + ".properties");
		}
		
		// Put all configuration into an ConfigObject
		this.getPropertiesFromGui(config);
		
		// Save the configuration to a file
		try
		{
			FileWriter out = new FileWriter (config.getProperty("FileName"));
			config.store(out, "");
		}
		catch(IOException e)
		{
			alert = new MessageBox(shell, SWT.OK);
			alert.setMessage("Error writing file: " + config.getProperty("FileName"));
			alert.open();
			return;
		}
		
		alert = new MessageBox(shell, SWT.OK);
		alert.setMessage("Configuration saved successfully to file: " + config.getProperty("FileName"));
		alert.open();
		return;
	}
	
	/**
	 * Put the configuration from the GUI into the properties Object
	 * @param config The Properties object where the current configuration values 
	 * 				should be stored in.
	 */
	private void getPropertiesFromGui(Properties config)
	{
		config.setProperty("ConfigurationName", configurations.getText());
		
		config.setProperty("Database.columnInfoTable", dbTable.getText());		
		config.setProperty("Database.password", dbPassword.getText());
		config.setProperty("Database.user", dbUser.getText());
		config.setProperty("Database.connectString", dbConnect.getText());
		
		config.setProperty("ForeignKeyFinder.checkSchema", schema.getText());
		config.setProperty("ForeignKeyFinder.schemaName", spiderOutSchema.getText());
		config.setProperty("ForeignKeyFinder.tableName", spiderOutTable.getText());
		
		StringBuffer heuristics = new StringBuffer();
		StringBuffer emphasises = new StringBuffer();
		if(numberOfValuesButton.getSelection())
		{
			heuristics.append("ForeignKeyHeuristicMinNumberDependendValues");
			emphasises.append(EmphasisNumberOfValues.getText());
		}
		if(coveringButton.getSelection())
		{
			if(heuristics.length() > 0)
			{
				heuristics.append(",");
				emphasises.append(",");
			}
			heuristics.append("ForeignKeyHeuristicCovering");
			emphasises.append(EmphasiseCovering.getText());
		}
		if(numberOfDependenciesButton.getSelection())
		{
			if(heuristics.length() > 0)
			{
				heuristics.append(",");
				emphasises.append(",");
			}
			heuristics.append("ForeignKeyHeuristicNumberOfReferences");
			emphasises.append(EmphasisNumberOfDep.getText());
		}
		if(columnNameButton.getSelection())
		{
			if(heuristics.length() > 0)
			{
				heuristics.append(",");
				emphasises.append(",");
			}
			heuristics.append("ForeignKeyHeuristicColumnName");
			emphasises.append(EmphasisCloumnName.getText());
		}
		if(minMaxButton.getSelection())
		{
			if(heuristics.length() > 0)
			{
				heuristics.append(",");
				emphasises.append(",");
			}
			heuristics.append("ForeignKeyHeuristicSpreadingOverReference");
			emphasises.append(EmphasisMinMax.getText());
		}
		config.setProperty("ForeignKeyFinder.HeuristicsToUse", heuristics.toString());
		config.setProperty("ForeignKeyFinder.HeuristicsEmphasis", emphasises.toString());
		
		// heuristic config
		config.setProperty("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.minNumberOfValues", 
				configMinNumberOfValues.getText());
		config.setProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringMin", 
				configCoveringMin.getText());
		config.setProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringFalse", 
				configCoveringFalse.getText());
		config.setProperty("Heuristics.ForeignKeyHeuristicNumberOfReferences.MaxCount", 
				configNoDMax.getText());
		config.setProperty("Heuristics.ForeignKeyHeuristicSpreadingOverReference.ValuesOutsideMinMaxPercentage", 
				configMMValuesOutside.getText());
		config.setProperty("Heuristics.ForeignKeyHeuristicSpreadingOverReference.CheckColumnInfo", 
				(new Boolean(configMMCheckColumnInfo.getSelection())).toString());
		config.setProperty("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.CheckColumnInfo", 
				(new Boolean(configViACheckColumnInfo.getSelection())).toString());
		
		// extended configuration
		config.setProperty("ForeignKeyFinder.writeResultIntoDatabase", 
				new Boolean(saveResultInDatabase.getSelection()).toString());
		config.setProperty("ForeignKeyFinder.printResultsToShell", 
				new Boolean(printResultToShell.getSelection()).toString());
		config.setProperty("Logger.Level", Loglevel.getText());
		
	}

}
