/**
 * The progress window that is shown while searching for ForeignKeys in the inclusion dependencies.
 * If there is an error while searching for FK's, a messagebox is shown.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.fkfinderGui;

import java.util.Properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKeyFinderExec;

/**
 * ForeignKeyFinderResult
 *
 */
public class FkfinderGuiWorking {

	private Label status;
	private Label label_1;
	private ProgressBar progressBar_1;
	private ProgressBar progressBar;
	private Label label;
	protected Shell shell;
	private Display display;
	
	/**
	 * The Numbers for the status labels are stored here.
	 */
	private double ready1;
	private double overall1;
	private double ready2;
	private double overall2;
	private String statusText;
	private String messageText;
	
	/**
	 * The finderobject that runs the search.
	 */
	private ForeignKeyFinderExec finder;
	
	/**
	 * Launch the application
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			FkfinderGuiWorking window = new FkfinderGuiWorking();
			window.open(false, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Open the window
	 */
	public void open(boolean execFinder, Properties config) {
		this.display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		if(execFinder)
		{
			this.startFinder(config);
		}
		while (!shell.isDisposed()) {
			if (!this.display.readAndDispatch())
				display.sleep();
		}
	}

	/**
	 * Create contents of the window
	 */
	protected void createContents() {
		shell = new Shell();
		shell.setSize(377, 183);
		shell.setText("FKFinder - progress");

		progressBar = new ProgressBar(shell, SWT.SMOOTH);
		progressBar.setBounds(10, 150, 357, 20);

		label = new Label(shell, SWT.NONE);
		label.setText("0 out of 0 IND's finished");
		label.setBounds(10, 130, 357, 14);

		progressBar_1 = new ProgressBar(shell, SWT.SMOOTH);
		progressBar_1.setBounds(10, 90, 357, 20);

		label_1 = new Label(shell, SWT.NONE);
		label_1.setText("0 out of 0 Heuristics ready");
		label_1.setBounds(10, 70, 357, 14);

		final Group statusGroup = new Group(shell, SWT.NONE);
		statusGroup.setText("Status");
		statusGroup.setBounds(10, 10, 357, 49);

		status = new Label(statusGroup, SWT.NONE);
		status.setBounds(10, 20,337, 14);
		status.setText("setting up DB connection");
		//
	}
	
	/**
	 * Starts the Finder with a given configuration.
	 * The Finder is started as seperate thread.
	 * @param config The Properties object that should be used for the run.
	 */
	public void startFinder(Properties config)
	{
		shell.forceFocus();
		this.finder = new ForeignKeyFinderExec(config);
		finder.setUpdateGui(this);
		finder.start();
	}
	
	/**
	 * This method is called when there was an error while running the script.
	 * @param message The Message that is shown on the GUI.
	 * 				Should be enough information for the User to know what happend.
	 */
	public void errorOccured(String message)
	{
		this.messageText = message;
		this.display.asyncExec(new Runnable() {
	        public void run() {
		MessageBox alert = new MessageBox(shell, SWT.OK);
		alert.setMessage("Error while running FKFinder: " + messageText);
		int val = alert.open();
		if(val == SWT.OK)
		{
			shell.close();
		}
	        }});
	}
	
	/**
	 * Sets the Heuristic progressbar and label
	 * @param ready
	 * @param overall
	 */
	public void updateHeuristics(int ready, int overall, String statusT)
	{
		this.ready1 = ready;
		this.overall1 = overall;
		this.statusText = statusT;
		this.display.asyncExec(new Runnable() {
	        public void run() {
	            progressBar_1.setSelection((int)(Math.round(ready1 / overall1 * 100)));
	            label_1.setText((int)ready1 + " out of "+(int)overall1+" Heuristics ready");
	            status.setText(statusText);
	          }});
	}
	
	/**
	 * Set the Ind Progress bar and label
	 * @param ready
	 * @param overall
	 */
	public void updateInds(int ready, int overall)
	{
		this.ready2 = ready;
		this.overall2 = overall;
		this.display.asyncExec(new Runnable() {
	        public void run() {
	        	try {
		            Thread.sleep(100);
		          } catch (Throwable th) {
		          }
	            progressBar.setSelection((int)(Math.round(ready2 / overall2 * 100)));
	            label.setText((int)ready2 + " out of "+(int)overall2+" IND's ready");
	          }});
	}
	
	/**
	 * This method should be called from the finder after finishing the run.
	 * It loads the result window and closes the progress window. 
	 */
	public void finderFinished()
	{
		this.display.asyncExec(new Runnable() {
	        public void run() {
	        	try {
		            Thread.sleep(100);
		          } catch (Throwable th) {
		          }
		  		FkFinderGuiResult result = new FkFinderGuiResult();
				result.setResult(finder);
				result.run();
	          }});
		this.display.asyncExec(new Runnable() {
	        public void run() {
	        	try {
		            Thread.sleep(100);
		          } catch (Throwable th) {
		          }
		  		shell.dispose();
	          }});
		
	}

}
