/**
 * This heuristic compares the name of the dependend attribut with several values from the
 * referenced column.
 * 
 * The idea of this heuristc is, that in some cases the name of the referneced column/table
 * is included in the name of the column of the dependend column
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicColumnName extends ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicColumnName(Connection conn, Properties config,
			Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		HeuristicResultValue returnValue = HeuristicResultValue.UNKNOWN;
		String returnDescription = "No Matching in Names";
		
		// comparing ColumnNames
		if(key.getDependentColumn().trim().equalsIgnoreCase(key.getReferencedColumn().trim()))
		{
			returnValue = HeuristicResultValue.TRUE;
			returnDescription = "Matching in Columnname";
		}
		// comparing Coulmnname with TableName
		else if(key.getDependentColumn().trim().equalsIgnoreCase(key.getReferencedTable().trim()))
		{
			returnValue = HeuristicResultValue.TRUE;
			returnDescription = "Matching in between Column and Tablename";
		}
		// checking if one Strinbg is uncluded in the other
		else if(key.getDependentColumn().toLowerCase().indexOf(key.getReferencedColumn().toLowerCase()) > -1)
		{
			returnValue = HeuristicResultValue.TRUE;
			returnDescription = "Referenced columnname included in dependend Columnname";
		}
		else if(key.getReferencedColumn().toLowerCase().indexOf(key.getDependentColumn().toLowerCase()) > -1)
		{
			returnValue = HeuristicResultValue.TRUE;
			returnDescription = "Dependend columnname included in referenced Columnname";
		}
		else if(key.getDependentColumn().toLowerCase().indexOf(key.getReferencedTable().toLowerCase()) > -1)
		{
			returnValue = HeuristicResultValue.TRUE;
			returnDescription = "Referenced tablename included in dependen Columnname";
		}
		
		return new HeuristicResult(returnValue, returnDescription);
	}

}
