/**
 * This Heuristics calculates the Covering between dependend and referenced Attribute.
 * If the value is below the configured percentage, the IND is no foreign key.
 * 
 * Idea of the Heuristic is, that if the values in the dependend Atribute did not cover
 * X% of the values in the referenced Attribute it does not seem to be an Foreign Key.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicCovering extends ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 * @see ForeignKeyHeuristic()
	 */
	public ForeignKeyHeuristicCovering(Connection conn, Properties config,
			Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		this.properties.add("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringMin");
		this.properties.add("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringFalse");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		this.logger.debug("Calculating for Column " + key.getDependentColumn());

		// check if needed properties are set and stop if not
		if(!this.checkProperties())
		{
			return new HeuristicResult();
		}
		
		float percentageCoveringTrue = new Long(
				this.config.getProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringMin")).longValue();
		float percentageCoveringFalse = new Long(
				this.config.getProperty("Heuristics.ForeignKeyHeuristicCovering.PercentageCoveringFalse")).longValue();
		try
		{
			// Number Of different Values in depend Attribut
			StringBuffer request = new StringBuffer(200);
			request.append("SELECT count(DISTINCT \"").append(key.getDependentColumn()).append("\") FROM ")
				.append(key.getSchemaName()).append(".\"").append(key.getDependentTable()).append("\"");
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			float valuesDependend = result.getLong(1);
			
			// Verbindung schliessen
			result.close();
			stmt.close();
			
			// Number of Values in referenced attribute
			request = new StringBuffer(200);
			request.append("SELECT count(DISTINCT \"").append(key.getReferencedColumn()).append("\") FROM ")
				.append(key.getSchemaName()).append(".\"").append(key.getReferencedTable()).append("\"");
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			float valuesReferenced = result.getLong(1);
			
			// calculating Heuristic Value
			HeuristicResultValue resultValue = HeuristicResultValue.UNKNOWN;
			float resultPercentage = (valuesDependend / valuesReferenced) * 100;
						
			if(resultPercentage >= percentageCoveringTrue)
			{
				resultValue = HeuristicResultValue.TRUE;
			}
			else if(resultPercentage < percentageCoveringFalse)
			{
				resultValue = HeuristicResultValue.FALSE;
			}
			//  Verbindung schliessen
			result.close();
			stmt.close();
			
			this.logger.debug("Set Probability for Column " + key.getDependentColumn() + ": " + resultValue);
			return new HeuristicResult(resultValue, "Covering percentage: " + resultPercentage + "%");
			
		}catch (SQLException e)
		{
			logger.fatal("Could not calculate covering percentage for Key:" + key.toString());
			e.printStackTrace();
		}
		
		// calculates the Percentage of covering between dependend and referenced attribute
		return new HeuristicResult();
	}

}
