/**
 * This Heuristics counts the number of different values in the dependend attribute.
 * 
 * Idea of this heuristic is, that if the number of different values in the dependend attribute
 * is below a configured value X, the IND is no Foreign Key.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.util.Properties;
import java.util.Vector;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicMinNumberDependendValues extends ForeignKeyHeuristic {

	/**
	 * 
	 * @see ForeignKeyHeuristic()
	 */
	public ForeignKeyHeuristicMinNumberDependendValues(Connection conn, Properties config, 
			Vector<ForeignKey> foreignKeys)
	{
		super(conn, config, foreignKeys);
		this.properties.add("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.minNumberOfValues");
		this.properties.add("Database.columnInfoTable");
		this.properties.add("Heuristics.ForeignKeyHeuristicMinNumberDependendValues.CheckColumnInfo");
	}
	
	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) 
	{
		this.logger.debug("Calculating for Column " + key.getDependentColumn());
		
		// check if needet properties are set and stop if not
		if(!this.checkProperties())
		{
			return new HeuristicResult();
		}
		int minNumberOfValues = new Integer(
				this.config.getProperty(
						"Heuristics.ForeignKeyHeuristicMinNumberDependendValues.minNumberOfValues"))
				.intValue();
		boolean useColumnInfo = 
			new Boolean(this.config.getProperty(
					"Heuristics.ForeignKeyHeuristicMinNumberDependendValues.CheckColumnInfo"));
		try
		{
			StringBuffer request = new StringBuffer(200);
			if(useColumnInfo)
			{
				// Check Distinct Values in columninfo table
				request.append("Select NUM_DISTINCT_COLUMN FROM ")
					.append(this.config.getProperty("Database.columnInfoTable"))
					.append(" WHERE COLUMN_NAME = '").append(key.getDependentColumn()).append("'")
					.append(" AND TABLE_NAME = '").append(key.getDependentTable()).append("'");
			}
			else
			{
				// check distinct values with direct request
				request.append("SELECT count(DISTINCT \"").append(key.getDependentColumn()).append("\") FROM ")
					.append(key.getSchemaName()).append(".\"").append(key.getDependentTable()).append("\"");
			}
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			
			long numberOfValues = result.getLong(1);
			HeuristicResultValue resultValue = HeuristicResultValue.FALSE;
			if(numberOfValues >= minNumberOfValues)
			{
				resultValue = HeuristicResultValue.UNKNOWN;
			}
			//  Verbindung schliessen
			result.close();
			stmt.close();
			
			this.logger.debug("Set Probability for Column " + key.getDependentColumn() + ": " + resultValue);
			return new HeuristicResult(resultValue, "Number of distinct values: " + numberOfValues);
		}
		catch(SQLException e)
		{
			this.logger.fatal("Could not read Dependencie: " + key.toString());
			e.printStackTrace();
		}
		
		return new HeuristicResult();
	}

}
