/**
 * This Heuristic calculates the number of referenced Attributs from one dependend.
 * 
 *  The Idea of this heuristic is, taht if the counted number is higher than X, the IND is
 *  no Foreign Key.
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicNumberOfReferences extends
		ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicNumberOfReferences(Connection conn,
			Properties config, Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		this.properties.add("Heuristics.ForeignKeyHeuristicNumberOfReferences.MaxCount");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) 
	{
		int referenceCounter = 0;
		int maxCounter = new Integer(
				this.config.getProperty("Heuristics.ForeignKeyHeuristicNumberOfReferences.MaxCount"))
				.intValue();
		HeuristicResultValue returnValue = HeuristicResultValue.FALSE;
		for(ForeignKey key2 : this.foreignKeys)
		{
			if(!key.equals(key))
			{
				// check only if the keys are not equal
				if(key.getDependentColumn().equalsIgnoreCase(key2.getDependentColumn()) 
						&& key.getDependentTable().equalsIgnoreCase(key2.getDependentTable()))
				{
					referenceCounter++;
				}
			}
		}
		if(referenceCounter <= maxCounter)
		{
			returnValue = HeuristicResultValue.UNKNOWN;
		}
		return new HeuristicResult(returnValue, "Count of other references: " + referenceCounter);
	}

}
