/**
 * This heuristic counts the number of values of the referenced attribute in a area that 
 * is givin by max() and min() of the dependend Attribut.
 * 
 * The Idea of the heuristic is, that the number of values in the range of max(dependen att) and
 * min(dependend att) is nearly the same in both tables
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 *@author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicPropotionalValueAmount extends
		ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicPropotionalValueAmount(Connection conn,
			Properties config, Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		
		this.properties.add("Heuristics.ForeignKeyHeuristicPropotionalValueAmount.maxDifferencePercent");
		this.properties.add("Heuristics.ForeignKeyHeuristicPropotionalValueAmount.CheckColumnInfo");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		this.logger.debug("Calculating for Column " + key.getDependentColumn());
		
		// check if needet properties are set and stop if not
		if(!this.checkProperties())
		{
			return new HeuristicResult();
		}
		int maxDifference = new Integer(this.config.getProperty(
					"Heuristics.ForeignKeyHeuristicPropotionalValueAmount.maxDifferencePercent")).intValue();
		boolean useColumnInfo = new Boolean(this.config.getProperty(
				"Heuristics.ForeignKeyHeuristicPropotionalValueAmount.CheckColumnInfo"));
		try
		{
			StringBuffer request = new StringBuffer();
			// Count the number of values in the referenced Column that are metween
			//   min(dependen.column) and max(dependend.column)
			
			request.append("SELECT count(DISTINCT \"").append(key.getReferencedColumn())
				.append("\") FROM ").append(key.getSchemaName()).append(".\"")
				.append(key.getReferencedTable()).append("\"")
				.append(" WHERE (\"").append(key.getReferencedColumn()).append("\"").append(" <= ")
				.append("(SELECT max(\"").append(key.getDependentColumn()).append("\")")
				.append(" FROM ").append(key.getSchemaName()).append(".")
				.append("\"").append(key.getDependentTable()).append("\")")
				.append(") OR (\"").append(key.getReferencedColumn()).append("\" >= ")
				.append("(SELECT min(\"").append(key.getDependentColumn()).append("\")")
				.append(" FROM ").append(key.getSchemaName()).append(".")
				.append("\"").append(key.getDependentTable()).append("\"))");
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			
			float numberOfValuesInReferenced = result.getFloat(1);

			// Verbindung schliessen
			result.close();
			stmt.close();
			
			// count the number of values in the dependend Column
			request = new StringBuffer();
			if(useColumnInfo)
			{
				request.append("SELECT NUM_DISTINCT_COLUMN FROM ")
					.append(this.config.getProperty("Database.columnInfoTable"))
					.append(" WHERE COLUMN_NAME = '").append(key.getDependentColumn()).append("'")
					.append(" AND TABLE_NAME = '").append(key.getDependentTable()).append("'");
			}
			else
			{
				request.append("SELECT count(distinct \"").append(key.getDependentColumn())
					.append("\" FROM ").append(key.getSchemaName()).append(".\"")
					.append(key.getDependentTable()).append("\"");
			}
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			
			float numberOfValuesInDependend = result.getFloat(1);
			
			// calculate the diffference in the values.
			//   allowed are X% of the number in the dependend Column
			HeuristicResultValue resultValue = HeuristicResultValue.FALSE;
			if(Math.abs(numberOfValuesInDependend - numberOfValuesInReferenced) 
					< ((numberOfValuesInDependend / 100) * maxDifference))
			{
				resultValue = HeuristicResultValue.TRUE;
			}
			this.logger.debug("Set Probability for Column " + key.getDependentColumn() + ": " + resultValue);
			
			// Verbindung schliessen
			result.close();
			stmt.close();
			
			return new HeuristicResult(resultValue, "Number of values in Dependend: " 
					+ numberOfValuesInDependend + "; in Referenced: " + numberOfValuesInReferenced
					+ " allowed Difference: " + ((numberOfValuesInDependend / 100) * maxDifference));
		}
		catch(SQLException e)
		{
			this.logger.fatal("Could not read Dependencie: " + key.toString());
			e.printStackTrace();
		}
		
		return new HeuristicResult();
	}

}
