/**
 * This Heuristics calculates the Difference between the average length of the values
 * between depend and referenced column.
 * 
 * The Idea of this Heuristic is, that the average length of the values are nearly the same
 * in a Foreign key connection. If the value is higher than a configured parameter, 
 * it is no foreign key
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicPropotionalValueLength extends ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicPropotionalValueLength(Connection conn, Properties config,
			Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		this.properties.add("Heuristics.ForeignKeyHeuristicPropotionalValueLength.MaxDifferece");
		this.properties.add("Heuristics.ForeignKeyHeuristicPropotionalValueLength.MinLength");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		
		HeuristicResultValue returnValue = HeuristicResultValue.FALSE;
		String returnDescription = "";
		
		float maxDifference = new Float(
				this.config.getProperty("Heuristics.ForeignKeyHeuristicPropotionalValueLength.MaxDifferece"));
		float minlength = new Float(
				this.config.getProperty("Heuristics.ForeignKeyHeuristicPropotionalValueLength.MinLength"));
		
		// reading avarage length from both colums
		try
		{
			// dependend column
			StringBuffer request = new StringBuffer(200);
			request.append("SELECT avg(length(\"").append(key.getDependentColumn()).append("\")) FROM ")
				.append(key.getSchemaName()).append(".\"").append(key.getDependentTable()).append("\"");
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			float lenghtDependend = result.getFloat(1);
			
			// Verbindung schliessen
			result.close();
			stmt.close();
			
			// referenced column
			request = new StringBuffer(200);
			request.append("SELECT avg(length(\"").append(key.getReferencedColumn()).append("\")) FROM ")
			.append(key.getSchemaName()).append(".\"").append(key.getReferencedTable()).append("\"");
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			float lenghtReferenced = result.getFloat(1);
			
			// calculating the difference
			if(lenghtDependend > minlength && lenghtReferenced > minlength)
			{
				if(Math.abs(lenghtDependend - lenghtReferenced) > maxDifference)
				{
					returnValue = HeuristicResultValue.FALSE;
					returnDescription = "Average length does not match. dep: " 
						+ lenghtDependend + " Ref: " + lenghtReferenced;
				}
				else
				{
					returnValue = HeuristicResultValue.TRUE;
					returnDescription = "Average length does match. dep: " 
						+ lenghtDependend + " Ref: " + lenghtReferenced;
				}
			}
			else
			{
				returnValue = HeuristicResultValue.UNKNOWN;
				returnDescription = "Average length below minLength " 
					+ minlength + ".";
			}
			// Verbindung schliessen
			result.close();
			stmt.close();
		}
		catch(SQLException e)
		{
			returnDescription = "Error while calculating the difference";
			this.logger.error(e.getMessage());
		}
		
		return new HeuristicResult(returnValue, returnDescription);
	}

}
