/**
 * 
 */
package de.huberlin.informatik.wbi.aladin.intraschema.heuristics;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.Vector;

import de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResult;
import de.huberlin.informatik.wbi.aladin.util.HeuristicResultValue;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class ForeignKeyHeuristicShortTables extends ForeignKeyHeuristic {

	/**
	 * @param conn
	 * @param config
	 * @param foreignKeys
	 */
	public ForeignKeyHeuristicShortTables(Connection conn, Properties config,
			Vector<ForeignKey> foreignKeys) {
		super(conn, config, foreignKeys);
		
		this.properties.add("Heuristics.ForeignKeyHeuristicShortTables.CheckColumnInfo");
		this.properties.add("Heuristics.ForeignKeyHeuristicShortTables.MinNumberOfValues");
	}

	/* (non-Javadoc)
	 * @see de.huberlin.informatik.wbi.aladin.intraschema.heuristics.ForeignKeyHeuristic#calculateProbability(de.huberlin.informatik.wbi.aladin.intraschema.ForeignKey)
	 */
	@Override
	public HeuristicResult calculateProbability(ForeignKey key) {
		boolean useColumnInfo = new Boolean(this.config.getProperty(
				"Heuristics.ForeignKeyHeuristicShortTables.CheckColumnInfo"));
		int minNumberOfValues = new Integer(this.config.getProperty(
				"Heuristics.ForeignKeyHeuristicShortTables.MinNumberOfValues")).intValue();
		try
		{
			StringBuffer request = new StringBuffer();
			
			// Count values of referenced Attribut
			if(useColumnInfo)
			{
				request.append("SELECT NUM_DISTINCT_COLUMN FROM ")
					.append(this.config.getProperty("Database.columnInfoTable"))
					.append(" WHERE SCHEMA_NAME = '").append(key.getSchemaName())
					.append("' AND TABLE_NAME = '").append(key.getReferencedTable())
					.append("' AND COLUMN_NAME = '").append(key.getReferencedColumn())
					.append("'");
			}
			else
			{
				request.append("SELECT count(DISTINCT \"").append(key.getReferencedColumn())
					.append("\") FROM ").append(key.getSchemaName()).append(".\"")
					.append(key.getReferencedTable()).append("\"");
			}
			this.logger.debug(request.toString());
			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery(request.toString());
			result.next();
			
			long countReferenced = result.getLong(1);
			
			stmt.close();
			result.close();
			
			request = new StringBuffer();
			// count values of Dependend Attribut
			if(useColumnInfo)
			{
				request.append("SELECT NUM_DISTINCT_COLUMN FROM ")
					.append(this.config.getProperty("Database.columnInfoTable"))
					.append(" WHERE SCHEMA_NAME = '").append(key.getSchemaName())
					.append("' AND TABLE_NAME = '").append(key.getDependentTable())
					.append("' AND COLUMN_NAME = '").append(key.getDependentColumn())
					.append("'");
			}
			else
			{
				request.append("SELECT count(DISTINCT \"").append(key.getDependentColumn())
					.append("\") FROM ").append(key.getSchemaName()).append(".\"")
					.append(key.getDependentTable()).append("\"");
			}
			this.logger.debug(request.toString());
			stmt = conn.createStatement();
			result = stmt.executeQuery(request.toString());
			result.next();
			
			long countDependent = result.getLong(1);
			HeuristicResultValue resultValue = HeuristicResultValue.FALSE;
			if(countReferenced > minNumberOfValues && countDependent > minNumberOfValues)
			{
				resultValue = HeuristicResultValue.TRUE;
			}
		
			// Verbindung schliessen
			result.close();
			stmt.close();
			
			return new HeuristicResult(resultValue, "Number of values ref: " 
					+ countReferenced + " Number of values dep: " + countDependent);
		
		}
		catch(SQLException e)
		{
			this.logger.fatal("Could not read Dependencie: " + key.toString());
			e.printStackTrace();
		}
		
		return new HeuristicResult();
	}

}
