/**
 * This class holds the Result of a heuristic
 */
package de.huberlin.informatik.wbi.aladin.util;

import java.io.Serializable;

/**
 * @author Oliver Albrecht (oliver.albrecht@informatik.hu-berlin.de)
 *
 */
public class HeuristicResult implements Serializable {
	
	private static final long serialVersionUID = -3194983570413611178L;
	/**
	 * Holds the Value of one heuristic
	 */
	private HeuristicResultValue result;
	
	/**
	 * Holds the Description of the result
	 */
	private String description;
	
	/**
	 * holds the emphasis for the heuristic that was tested.
	 */
	private int emphasis = 0;
	
	/**
	 * Construtor for a new Heuriustc Result
	 * @param result Says if the IND is a foreign Key or not
	 * @param description Describes the Result
	 */
	public HeuristicResult(HeuristicResultValue result, String description)
	{
		this.result = result;
		this.description = description;
	}
	
	/**
	 * Contructor called in case of an error
	 *
	 */
	public HeuristicResult()
	{
		this.result = HeuristicResultValue.FALSE;
		this.description = "error";
	}
	
	/**
	 * @see Object#toString()
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		
		sb.append(this.result).append(", ").append(this.description);
		
		return sb.toString();
	}
	
	/**
	 * returns if the key is an foreign key
	 * @return
	 */
	public boolean isForeignKey()
	{
		return (this.result == HeuristicResultValue.TRUE);
	}
	
	/**
	 * Returns if the key is NO foreign key.
	 * @return
	 */
	public boolean isNoForeignKey()
	{
		return (this.result == HeuristicResultValue.FALSE);
	}
	
	/**
	 * Returns if the Result of the Heuristic is clear or unknown. 
	 * @return true, if the result was clear, false if it was unknown
	 */
	public boolean hasClearResult()
	{
		return (this.result != HeuristicResultValue.UNKNOWN);
	}
	
	/**
	 * Sets the Emphasis of the heuristic.
	 * @param emphasis
	 */
	public void setEmphasis(int emphasis)
	{
		this.emphasis = emphasis;
	}
	
	/**
	 * Returns the Value for the emphasis.
	 * @return
	 */
	public int getEmphasis()
	{
		return this.emphasis;
	}
}
